@@ -1,7 +1,7 @@
-from pybamm import exp
+from pybamm import exp, constants
 
 
-def lico2_diffusivity_Dualfoil1998(sto, T, T_inf, E_D_s, R_g):
+def lico2_diffusivity_Dualfoil1998(sto, T):
     """
     LiCo2 diffusivity as a function of stochiometry, in this case the
     diffusivity is taken to be a constant. The value is taken from Dualfoil [1].
@@ -12,24 +12,18 @@ def lico2_diffusivity_Dualfoil1998(sto, T, T_inf, E_D_s, R_g):
 
     Parameters
     ----------
-    sto: :class: `numpy.Array`
+    sto: :class:`pybamm.Symbol`
         Electrode stochiometry
-    T: :class: `numpy.Array`
+    T: :class:`pybamm.Symbol`
         Dimensional temperature
-    T_inf: double
-        Reference temperature
-    E_D_s: double
-        Solid diffusion activation energy
-    R_g: double
-        The ideal gas constant
 
     Returns
     -------
-    : double
+    :class:`pybamm.Symbol`
         Solid diffusivity
     """
-
     D_ref = 1 * 10 ** (-13)
-    arrhenius = exp(E_D_s / R_g * (1 / T_inf - 1 / T))
+    E_D_s = 18550
+    arrhenius = exp(E_D_s / constants.R * (1 / 298.15 - 1 / T))
 
     return D_ref * arrhenius
