@@ -13,7 +13,7 @@ def lico2_entropic_change_Moura2016(sto, c_p_max):
 
           Parameters
           ----------
-          sto: double
+          sto : :class:`pybamm.Symbol`
                Stochiometry of material (li-fraction)
 
     """
