@@ -1,7 +1,7 @@
-from pybamm import exp
+from pybamm import exp, constants
 
 
-def nca_diffusivity_Kim2011(sto, T, T_inf, E_D_s, R_g):
+def nca_diffusivity_Kim2011(sto, T):
     """
     NCA diffusivity as a function of stochiometry [1].
 
@@ -14,24 +14,19 @@ def nca_diffusivity_Kim2011(sto, T, T_inf, E_D_s, R_g):
 
     Parameters
     ----------
-    sto: :class: `numpy.Array`
+    sto: :class:`pybamm.Symbol`
         Electrode stochiometry
-    T: :class: `numpy.Array`
+    T: :class:`pybamm.Symbol`
         Dimensional temperature
-    T_inf: double
-        Reference temperature
-    E_D_s: double
-        Solid diffusion activation energy
-    R_g: double
-        The ideal gas constant
 
     Returns
     -------
-    : double
+    :class:`pybamm.Symbol`
         Solid diffusivity
     """
 
     D_ref = 3 * 10 ** (-15)
-    arrhenius = exp(E_D_s / R_g * (1 / T_inf - 1 / T))
+    E_D_s = 2e4
+    arrhenius = exp(E_D_s / constants.R * (1 / 298.15 - 1 / T))
 
     return D_ref * arrhenius
