@@ -1,7 +1,7 @@
-from pybamm import exp
+from pybamm import exp, constants
 
 
-def electrolyte_conductivity_Nyman2008(c_e, T, T_inf, E_k_e, R_g):
+def electrolyte_conductivity_Nyman2008(c_e, T):
     """
     Conductivity of LiPF6 in EC:EMC (3:7) as a function of ion concentration. The data
     comes from [1].
@@ -12,19 +12,14 @@ def electrolyte_conductivity_Nyman2008(c_e, T, T_inf, E_k_e, R_g):
     Electrochim. Acta, vol. 53, no. 22, pp. 6356–6365, 2008.
     Parameters
     ----------
-    c_e: :class: `numpy.Array`
+    c_e: :class:`pybamm.Symbol`
         Dimensional electrolyte concentration
-    T: :class: `numpy.Array`
+    T: :class:`pybamm.Symbol`
         Dimensional temperature
-    T_inf: double
-        Reference temperature
-    E_k_e: double
-        Electrolyte conductivity activation energy
-    R_g: double
-        The ideal gas constant
+
     Returns
     -------
-    :`numpy.Array`
+    :class:`pybamm.Symbol`
         Solid diffusivity
     """
 
@@ -32,6 +27,7 @@ def electrolyte_conductivity_Nyman2008(c_e, T, T_inf, E_k_e, R_g):
         0.1297 * (c_e / 1000) ** 3 - 2.51 * (c_e / 1000) ** 1.5 + 3.329 * (c_e / 1000)
     )
 
-    arrhenius = exp(E_k_e / R_g * (1 / T_inf - 1 / T))
+    E_k_e = 34700
+    arrhenius = exp(E_k_e / constants.R * (1 / 298.15 - 1 / T))
 
     return sigma_e * arrhenius
