@@ -1,7 +1,7 @@
-from pybamm import exp
+from pybamm import exp, constants
 
 
-def electrolyte_diffusivity_Nyman2008(c_e, T, T_inf, E_D_e, R_g):
+def electrolyte_diffusivity_Nyman2008(c_e, T):
     """
     Diffusivity of LiPF6 in EC:EMC (3:7) as a function of ion concentration. The data
     comes from [1]
@@ -12,23 +12,19 @@ def electrolyte_diffusivity_Nyman2008(c_e, T, T_inf, E_D_e, R_g):
     Electrochim. Acta, vol. 53, no. 22, pp. 6356–6365, 2008.
     Parameters
     ----------
-    c_e: :class: `numpy.Array`
+    c_e: :class:`pybamm.Symbol`
         Dimensional electrolyte concentration
-    T: :class: `numpy.Array`
+    T: :class:`pybamm.Symbol`
         Dimensional temperature
-    T_inf: double
-        Reference temperature
-    E_D_e: double
-        Electrolyte diffusion activation energy
-    R_g: double
-        The ideal gas constant
+
     Returns
     -------
-    :`numpy.Array`
+    :class:`pybamm.Symbol`
         Solid diffusivity
     """
 
     D_c_e = 8.794e-11 * (c_e / 1000) ** 2 - 3.972e-10 * (c_e / 1000) + 4.862e-10
-    arrhenius = exp(E_D_e / R_g * (1 / T_inf - 1 / T))
+    E_D_e = 37040
+    arrhenius = exp(E_D_e / constants.R * (1 / 298.15 - 1 / T))
 
     return D_c_e * arrhenius
