@@ -23,13 +23,22 @@ class BaseInterface(pybamm.BaseSubModel):
 
     def __init__(self, param, domain, reaction):
         super().__init__(param, domain)
-        self.reaction = reaction
         if reaction == "lithium-ion main":
             self.reaction_name = ""  # empty reaction name for the main reaction
+            self.Reaction_icd = "Interfacial current density"
         elif reaction == "lead-acid main":
             self.reaction_name = ""  # empty reaction name for the main reaction
+            self.Reaction_icd = "Interfacial current density"
         elif reaction == "lead-acid oxygen":
             self.reaction_name = " oxygen"
+            self.Reaction_icd = "Oxygen interfacial current density"
+        elif reaction == "lithium-ion oxygen":
+            self.reaction_name = " oxygen"
+            self.Reaction_icd = "Oxygen interfacial current density"
+        elif reaction == "sei":
+            self.reaction_name = " sei"
+            self.Reaction_icd = "Sei interfacial current density"
+        self.reaction = reaction
 
     def _get_exchange_current_density(self, variables):
         """
@@ -46,10 +55,10 @@ class BaseInterface(pybamm.BaseSubModel):
             The exchange current density.
         """
         c_e = variables[self.domain + " electrolyte concentration"]
+        T = variables[self.domain + " electrode temperature"]
 
         if self.reaction == "lithium-ion main":
             c_s_surf = variables[self.domain + " particle surface concentration"]
-            T = variables[self.domain + " electrode temperature"]
 
             # If variable was broadcast, take only the orphan
             if (
@@ -61,31 +70,35 @@ class BaseInterface(pybamm.BaseSubModel):
                 c_e = c_e.orphans[0]
                 T = T.orphans[0]
             if self.domain == "Negative":
-                prefactor = self.param.m_n(T) / self.param.C_r_n
+                j0 = self.param.j0_n(c_e, c_s_surf, T) / self.param.C_r_n
             elif self.domain == "Positive":
-                prefactor = self.param.gamma_p * self.param.m_p(T) / self.param.C_r_p
-            j0 = prefactor * (
-                c_e ** (1 / 2) * c_s_surf ** (1 / 2) * (1 - c_s_surf) ** (1 / 2)
-            )
+                j0 = (
+                    self.param.gamma_p
+                    * self.param.j0_p(c_e, c_s_surf, T)
+                    / self.param.C_r_p
+                )
 
         elif self.reaction == "lead-acid main":
             # If variable was broadcast, take only the orphan
-            if isinstance(c_e, pybamm.Broadcast):
+            if isinstance(c_e, pybamm.Broadcast) and isinstance(T, pybamm.Broadcast):
                 c_e = c_e.orphans[0]
+                T = T.orphans[0]
             if self.domain == "Negative":
-                j0 = self.param.j0_n_S_ref * c_e
+                j0 = self.param.j0_n(c_e, T)
             elif self.domain == "Positive":
-                c_w = self.param.c_w(c_e)
-                j0 = self.param.j0_p_S_ref * c_e ** 2 * c_w
+                j0 = self.param.j0_p(c_e, T)
 
         elif self.reaction == "lead-acid oxygen":
             # If variable was broadcast, take only the orphan
-            if isinstance(c_e, pybamm.Broadcast):
+            if isinstance(c_e, pybamm.Broadcast) and isinstance(T, pybamm.Broadcast):
                 c_e = c_e.orphans[0]
+                T = T.orphans[0]
             if self.domain == "Negative":
                 j0 = pybamm.Scalar(0)
             elif self.domain == "Positive":
-                j0 = self.param.j0_p_Ox_ref * c_e  # ** self.param.exponent_e_Ox
+                j0 = self.param.j0_p_Ox(c_e, T)
+        else:
+            j0 = pybamm.Scalar(0)
 
         return j0
 
@@ -142,6 +155,10 @@ class BaseInterface(pybamm.BaseSubModel):
                 ocp = self.param.U_p_Ox
             dUdT = pybamm.Scalar(0)
 
+        else:
+            ocp = pybamm.Scalar(0)
+            dUdT = pybamm.Scalar(0)
+
         return ocp, dUdT
 
     def _get_number_of_electrons_in_reaction(self):
@@ -153,6 +170,21 @@ class BaseInterface(pybamm.BaseSubModel):
                 return self.param.ne_p
         elif self.reaction == "lead-acid oxygen":
             return self.param.ne_Ox
+        else:
+            return pybamm.Scalar(0)
+
+    def _get_electrolyte_reaction_signed_stoichiometry(self):
+        "Returns the number of electrons in the reaction"
+        if self.reaction in ["lithium-ion main", "sei"]:
+            # Both the main reaction current contribute to the electrolyte reaction
+            # current
+            return pybamm.Scalar(1), pybamm.Scalar(1)
+        elif self.reaction == "lead-acid main":
+            return self.param.s_plus_n_S, self.param.s_plus_p_S
+        elif self.reaction == "lead-acid oxygen":
+            return self.param.s_plus_Ox, self.param.s_plus_Ox
+        else:
+            return pybamm.Scalar(0), pybamm.Scalar(0)
 
     def _get_delta_phi(self, variables):
         "Calculate delta_phi, and derived variables, using phi_s and phi_e"
@@ -189,11 +221,14 @@ class BaseInterface(pybamm.BaseSubModel):
             j_scale = i_typ / (self.param.a_p_dim * L_x)
 
         # Average, and broadcast if necessary
-        j_av = pybamm.x_average(j)
         if j.domain == []:
+            j_av = j
             j = pybamm.FullBroadcast(j, self.domain_for_broadcast, "current collector")
         elif j.domain == ["current collector"]:
+            j_av = j
             j = pybamm.PrimaryBroadcast(j, self.domain_for_broadcast)
+        else:
+            j_av = pybamm.x_average(j)
 
         variables = {
             self.domain
@@ -254,12 +289,26 @@ class BaseInterface(pybamm.BaseSubModel):
         return variables
 
     def _get_standard_whole_cell_interfacial_current_variables(self, variables):
-
+        """
+        Get variables associated with interfacial current over the whole cell domain
+        This function also automatically increments the "total source term" variables
+        """
         i_typ = self.param.i_typ
         L_x = self.param.L_x
         j_n_scale = i_typ / (self.param.a_n_dim * L_x)
         j_p_scale = i_typ / (self.param.a_p_dim * L_x)
 
+        j_n_av = variables[
+            "X-averaged negative electrode"
+            + self.reaction_name
+            + " interfacial current density"
+        ]
+        j_p_av = variables[
+            "X-averaged positive electrode"
+            + self.reaction_name
+            + " interfacial current density"
+        ]
+
         j_n = variables[
             "Negative electrode" + self.reaction_name + " interfacial current density"
         ]
@@ -270,20 +319,43 @@ class BaseInterface(pybamm.BaseSubModel):
         j = pybamm.Concatenation(j_n, j_s, j_p)
         j_dim = pybamm.Concatenation(j_n_scale * j_n, j_s, j_p_scale * j_p)
 
-        if self.reaction_name == "":
-            variables = {
-                "Interfacial current density": j,
-                "Interfacial current density [A.m-2]": j_dim,
-                "Interfacial current density per volume [A.m-3]": i_typ / L_x * j,
-            }
-        else:
-            reaction_name = self.reaction_name[1:].capitalize()
-            variables = {
-                reaction_name + " interfacial current density": j,
-                reaction_name + " interfacial current density [A.m-2]": j_dim,
-                reaction_name
-                + " interfacial current density per volume [A.m-3]": i_typ / L_x * j,
+        variables.update(
+            {
+                self.Reaction_icd: j,
+                self.Reaction_icd + " [A.m-2]": j_dim,
+                self.Reaction_icd + " per volume [A.m-3]": i_typ / L_x * j,
             }
+        )
+
+        s_n, s_p = self._get_electrolyte_reaction_signed_stoichiometry()
+        s = pybamm.Concatenation(
+            pybamm.FullBroadcast(s_n, "negative electrode", "current collector"),
+            pybamm.FullBroadcast(0, "separator", "current collector"),
+            pybamm.FullBroadcast(s_p, "positive electrode", "current collector"),
+        )
+        variables["Sum of electrolyte reaction source terms"] += s * j
+        variables["Sum of negative electrode electrolyte reaction source terms"] += (
+            s_n * j_n
+        )
+        variables["Sum of positive electrode electrolyte reaction source terms"] += (
+            s_p * j_p
+        )
+        variables[
+            "Sum of x-averaged negative electrode electrolyte reaction source terms"
+        ] += (s_n * j_n_av)
+        variables[
+            "Sum of x-averaged positive electrode electrolyte reaction source terms"
+        ] += (s_p * j_p_av)
+
+        variables["Sum of interfacial current densities"] += j
+        variables["Sum of negative electrode interfacial current densities"] += j_n
+        variables["Sum of positive electrode interfacial current densities"] += j_p
+        variables[
+            "Sum of x-averaged negative electrode interfacial current densities"
+        ] += j_n_av
+        variables[
+            "Sum of x-averaged positive electrode interfacial current densities"
+        ] += j_p_av
 
         return variables
 
@@ -297,13 +369,16 @@ class BaseInterface(pybamm.BaseSubModel):
             j_scale = i_typ / (self.param.a_p_dim * L_x)
 
         # Average, and broadcast if necessary
-        j0_av = pybamm.x_average(j0)
         if j0.domain == []:
+            j0_av = j0
             j0 = pybamm.FullBroadcast(
                 j0, self.domain_for_broadcast, "current collector"
             )
         elif j0.domain == ["current collector"]:
+            j0_av = j0
             j0 = pybamm.PrimaryBroadcast(j0, self.domain_for_broadcast)
+        else:
+            j0_av = pybamm.x_average(j0)
 
         variables = {
             self.domain
@@ -406,6 +481,30 @@ class BaseInterface(pybamm.BaseSubModel):
 
         return variables
 
+    def _get_standard_sei_film_overpotential_variables(self, eta_sei):
+
+        pot_scale = self.param.potential_scale
+        # Average, and broadcast if necessary
+        eta_sei_av = pybamm.x_average(eta_sei)
+        if eta_sei.domain == []:
+            eta_sei = pybamm.FullBroadcast(
+                eta_sei, self.domain_for_broadcast, "current collector"
+            )
+        elif eta_sei.domain == ["current collector"]:
+            eta_sei = pybamm.PrimaryBroadcast(eta_sei, self.domain_for_broadcast)
+
+        domain = self.domain.lower() + " electrode"
+        variables = {
+            self.domain + " electrode sei film overpotential": eta_sei,
+            "X-averaged " + domain + " sei film overpotential": eta_sei_av,
+            self.domain + " electrode sei film overpotential [V]": eta_sei * pot_scale,
+            "X-averaged "
+            + domain
+            + " sei film overpotential [V]": eta_sei_av * pot_scale,
+        }
+
+        return variables
+
     def _get_standard_surface_potential_difference_variables(self, delta_phi):
 
         if self.domain == "Negative":
@@ -415,13 +514,16 @@ class BaseInterface(pybamm.BaseSubModel):
         pot_scale = self.param.potential_scale
 
         # Average, and broadcast if necessary
-        delta_phi_av = pybamm.x_average(delta_phi)
         if delta_phi.domain == []:
+            delta_phi_av = delta_phi
             delta_phi = pybamm.FullBroadcast(
                 delta_phi, self.domain_for_broadcast, "current collector"
             )
         elif delta_phi.domain == ["current collector"]:
+            delta_phi_av = delta_phi
             delta_phi = pybamm.PrimaryBroadcast(delta_phi, self.domain_for_broadcast)
+        else:
+            delta_phi_av = pybamm.x_average(delta_phi)
 
         variables = {
             self.domain + " electrode surface potential difference": delta_phi,
@@ -459,13 +561,16 @@ class BaseInterface(pybamm.BaseSubModel):
         """
 
         # Average, and broadcast if necessary
-        ocp_av = pybamm.x_average(ocp)
         if ocp.domain == []:
+            ocp_av = ocp
             ocp = pybamm.FullBroadcast(
                 ocp, self.domain_for_broadcast, "current collector"
             )
         elif ocp.domain == ["current collector"]:
+            ocp_av = ocp
             ocp = pybamm.PrimaryBroadcast(ocp, self.domain_for_broadcast)
+        else:
+            ocp_av = pybamm.x_average(ocp)
         dUdT_av = pybamm.x_average(dUdT)
 
         if self.domain == "Negative":
@@ -494,8 +599,15 @@ class BaseInterface(pybamm.BaseSubModel):
             + " electrode"
             + self.reaction_name
             + " open circuit potential [V]": ocp_av_dim,
-            self.domain + " electrode entropic change": dUdT,
-            "X-averaged " + self.domain.lower() + " electrode entropic change": dUdT_av,
         }
+        if self.reaction_name == "":
+            variables.update(
+                {
+                    self.domain + " electrode entropic change": dUdT,
+                    "X-averaged "
+                    + self.domain.lower()
+                    + " electrode entropic change": dUdT_av,
+                }
+            )
 
         return variables
