@@ -5,8 +5,8 @@ import pybamm
 import pandas as pd
 import os
 import numbers
-import numpy as np
 from pprint import pformat
+from collections import defaultdict
 
 
 class ParameterValues:
@@ -73,8 +73,9 @@ class ParameterValues:
         if values is not None:
             # If base_parameters is a filename, load from that filename
             if isinstance(values, str):
-                path = os.path.split(values)[0]
-                values = self.read_parameters_csv(values)
+                file_path = self.find_parameter(values)
+                path = os.path.split(file_path)[0]
+                values = self.read_parameters_csv(file_path)
             else:
                 path = None
             # Don't check parameter already exists when first creating it
@@ -86,6 +87,13 @@ class ParameterValues:
     def __getitem__(self, key):
         return self._dict_items[key]
 
+    def get(self, key, default=None):
+        "Return item correspoonding to key if it exists, otherwise return default"
+        try:
+            return self._dict_items[key]
+        except KeyError:
+            return default
+
     def __setitem__(self, key, value):
         "Call the update functionality when doing a setitem"
         self.update({key: value})
@@ -108,6 +116,11 @@ class ParameterValues:
         "Get the items of the dictionary"
         return self._dict_items.items()
 
+    def copy(self):
+        """Returns a copy of the parameter values. Makes sure to copy the internal
+        dictionary."""
+        return ParameterValues(values=self._dict_items.copy())
+
     def search(self, key, print_values=True):
         """
         Search dictionary for keys containing 'key'.
@@ -121,19 +134,23 @@ class ParameterValues:
         Load standard set of components from a 'chemistry' dictionary
         """
         base_chemistry = chemistry["chemistry"]
-        # Create path to file
-        path = os.path.join(
-            pybamm.root_dir(), "pybamm", "input", "parameters", base_chemistry
-        )
+
         # Load each component name
-        for component_group in [
+
+        component_groups = [
             "cell",
             "anode",
             "cathode",
             "separator",
             "electrolyte",
             "experiment",
-        ]:
+        ]
+
+        # add sei parameters if provided
+        if "sei" in chemistry:
+            component_groups += ["sei"]
+
+        for component_group in component_groups:
             # Make sure component is provided
             try:
                 component = chemistry[component_group]
@@ -144,18 +161,20 @@ class ParameterValues:
                     )
                 )
             # Create path to component and load values
-            component_path = os.path.join(path, component_group + "s", component)
-            component_params = self.read_parameters_csv(
-                pybamm.get_parameters_filepath(
-                    os.path.join(component_path, "parameters.csv")
-                )
+            component_path = os.path.join(
+                base_chemistry, component_group + "s", component
             )
+            file_path = self.find_parameter(
+                os.path.join(component_path, "parameters.csv")
+            )
+            component_params = self.read_parameters_csv(file_path)
+
             # Update parameters, making sure to check any conflicts
             self.update(
                 component_params,
                 check_conflict=True,
                 check_already_exists=False,
-                path=component_path,
+                path=os.path.dirname(file_path),
             )
 
         # register (list of) citations
@@ -206,6 +225,8 @@ class ParameterValues:
         path : string, optional
             Path from which to load functions
         """
+        # check parameter values
+        self.check_parameter_values(values)
         # update
         for name, value in values.items():
             # check for conflicts
@@ -265,64 +286,33 @@ class ParameterValues:
                     values[name] = float(value)
             else:
                 self._dict_items[name] = value
-        # check parameter values
-        self.check_and_update_parameter_values(values)
         # reset processed symbols
         self._processed_symbols = {}
 
-    def check_and_update_parameter_values(self, values):
+    def check_parameter_values(self, values):
         # Make sure typical current is non-zero
         if "Typical current [A]" in values and values["Typical current [A]"] == 0:
             raise ValueError(
                 "'Typical current [A]' cannot be zero. A possible alternative is to "
                 "set 'Current function [A]' to `0` instead."
             )
-        if "C-rate" in values and "Current function [A]" in values:
+        if "C-rate" in values:
             raise ValueError(
-                "Cannot provide both 'C-rate' and 'Current function [A]' simultaneously"
+                "The 'C-rate' parameter has been deprecated, "
+                "use 'Current function [A]' instead. The cell capacity can be accessed "
+                "as 'Cell capacity [A.h]', and used to calculate current from C-rate."
             )
-        # If the capacity of the cell has been provided, make sure "C-rate" and current
-        # match with the stated capacity
-        if "Cell capacity [A.h]" in values or "Cell capacity [A.h]" in self._dict_items:
-            # Capacity from values takes precedence
-            if "Cell capacity [A.h]" in values:
-                capacity = values["Cell capacity [A.h]"]
-            else:
-                capacity = self._dict_items["Cell capacity [A.h]"]
-            # Make sure they match if both provided
-            # Update the other if only one provided
-            if "C-rate" in values:
-                # Can't provide C-rate as a function
-                if callable(values["C-rate"]):
-                    value = CrateToCurrent(values["C-rate"], capacity)
-                elif isinstance(values["C-rate"], tuple):
-                    data = values["C-rate"][1]
-                    current_data = np.stack([data[:, 0], data[:, 1] * capacity], axis=1)
-                    value = (values["C-rate"][0] + "_to_current", current_data)
-                elif values["C-rate"] == "[input]":
-                    value = CrateToCurrent(values["C-rate"], capacity, typ="input")
-                else:
-                    value = values["C-rate"] * capacity
-                self._dict_items["Current function [A]"] = value
-            elif "Current function [A]" in values:
-                if callable(values["Current function [A]"]):
-                    value = CurrentToCrate(values["Current function [A]"], capacity)
-                elif isinstance(values["Current function [A]"], tuple):
-                    data = values["Current function [A]"][1]
-                    c_rate_data = np.stack([data[:, 0], data[:, 1] / capacity], axis=1)
-                    value = (
-                        values["Current function [A]"][0] + "_to_Crate",
-                        c_rate_data,
-                    )
-                elif values["Current function [A]"] == "[input]":
-                    value = CurrentToCrate(
-                        values["Current function [A]"], capacity, typ="input"
-                    )
-                else:
-                    value = values["Current function [A]"] / capacity
-                self._dict_items["C-rate"] = value
-
-        return values
+        for param in values:
+            if "surface area density" in param:
+                raise ValueError(
+                    "Parameters involving 'surface area density' have been renamed to "
+                    "'surface area to volume ratio' ('{}' found)".format(param)
+                )
+            if "reaction rate" in param:
+                raise ValueError(
+                    "Parameters involving 'reaction rate' have been replaced with "
+                    "'exchange-current density' ('{}' found)".format(param)
+                )
 
     def process_model(self, unprocessed_model, inplace=True):
         """Assign parameter values to a model.
@@ -396,6 +386,10 @@ class ParameterValues:
         # Process timescale
         model.timescale = self.process_symbol(model.timescale)
 
+        # Process length scales
+        for domain, scale in model.length_scales.items():
+            model.length_scales[domain] = self.process_symbol(scale)
+
         pybamm.logger.info("Finish setting parameters for {}".format(model.name))
 
         return model
@@ -446,22 +440,22 @@ class ParameterValues:
 
         Parameters
         ----------
-        geometry : :class:`pybamm.Geometry`
-                Geometry specs to assign parameter values to
+        geometry : dict
+            Geometry specs to assign parameter values to
         """
         for domain in geometry:
-            for prim_sec_tabs, variables in geometry[domain].items():
+            for spatial_variable, spatial_limits in geometry[domain].items():
                 # process tab information if using 1 or 2D current collectors
-                if prim_sec_tabs == "tabs":
-                    for tab, position_size in variables.items():
+                if spatial_variable == "tabs":
+                    for tab, position_size in spatial_limits.items():
                         for position_size, sym in position_size.items():
-                            geometry[domain][prim_sec_tabs][tab][
+                            geometry[domain]["tabs"][tab][
                                 position_size
                             ] = self.process_symbol(sym)
                 else:
-                    for spatial_variable, spatial_limits in variables.items():
-                        for lim, sym in spatial_limits.items():
-                            geometry[domain][prim_sec_tabs][spatial_variable][
+                    for lim, sym in spatial_limits.items():
+                        if isinstance(sym, pybamm.Symbol):
+                            geometry[domain][spatial_variable][
                                 lim
                             ] = self.process_symbol(sym)
 
@@ -498,9 +492,12 @@ class ParameterValues:
                 # Scalar inherits name (for updating parameters) and domain (for
                 # Broadcast)
                 return pybamm.Scalar(value, name=symbol.name, domain=symbol.domain)
-            elif isinstance(value, pybamm.InputParameter):
-                value.domain = symbol.domain
-                return value
+            elif isinstance(value, pybamm.Symbol):
+                new_value = self.process_symbol(value)
+                new_value.domain = symbol.domain
+                return new_value
+            else:
+                raise TypeError("Cannot process parameter '{}'".format(value))
 
         elif isinstance(symbol, pybamm.FunctionParameter):
             new_children = [self.process_symbol(child) for child in symbol.children]
@@ -522,9 +519,21 @@ class ParameterValues:
             elif isinstance(function_name, pybamm.InputParameter):
                 # Replace the function with an input parameter
                 function = function_name
-            else:
+            elif (
+                isinstance(function_name, pybamm.Symbol)
+                and function_name.evaluates_to_number()
+            ):
+                # If the "function" provided is a pybamm scalar-like, use ones_like to
+                # get the right shape
+                function = function_name * pybamm.ones_like(*new_children)
+            elif callable(function_name):
                 # otherwise evaluate the function to create a new PyBaMM object
                 function = function_name(*new_children)
+            else:
+                raise TypeError(
+                    "Parameter provided for '{}' ".format(symbol.name)
+                    + "is of the wrong type (should either be scalar-like or callable)"
+                )
             # Differentiate if necessary
             if symbol.diff_variable is None:
                 function_out = function
@@ -599,32 +608,147 @@ class ParameterValues:
     def _ipython_key_completions_(self):
         return list(self._dict_items.keys())
 
+    def export_csv(self, filename):
 
-class CurrentToCrate:
-    "Convert a current function to a C-rate function"
+        # process functions and data to output
+        # like they appear in inputs csv files
+        parameter_output = {}
+        for key, val in self.items():
+            if callable(val):
+                val = "[function]" + val.__name__
+            elif isinstance(val, tuple):
+                val = "[data]" + val[0]
+            parameter_output[key] = [val]
 
-    def __init__(self, current, capacity, typ="function"):
-        self.current = current
-        self.capacity = capacity
-        self.type = typ
+        df = pd.DataFrame(parameter_output)
+        df = df.transpose()
+        df.to_csv(filename, header=None)
 
-    def __call__(self, t):
-        if self.type == "function":
-            return self.current(t) / self.capacity
-        elif self.type == "input":
-            return pybamm.InputParameter("Current function [A]") / self.capacity
+    def print_parameters(self, parameters, output_file=None):
+        """
+        Return dictionary of evaluated parameters, and optionally print these evaluated
+        parameters to an output file.
+        For dimensionless parameters that depend on the C-rate, the value is given as a
+        function of the C-rate (either x * Crate or x / Crate depending on the
+        dependence)
 
+        Parameters
+        ----------
+        parameters : class or dict containing :class:`pybamm.Parameter` objects
+            Class or dictionary containing all the parameters to be evaluated
+        output_file : string, optional
+            The file to print parameters to. If None, the parameters are not printed,
+            and this function simply acts as a test that all the parameters can be
+            evaluated, and returns the dictionary of evaluated parameters.
 
-class CrateToCurrent:
-    "Convert a C-rate function to a current function"
+        Returns
+        -------
+        evaluated_parameters : defaultdict
+            The evaluated parameters, for further processing if needed
 
-    def __init__(self, Crate, capacity, typ="function"):
-        self.Crate = Crate
-        self.capacity = capacity
-        self.type = typ
+        Notes
+        -----
+        A C-rate of 1 C is the current required to fully discharge the battery in 1
+        hour, 2 C is current to discharge the battery in 0.5 hours, etc
+        """
+        # Set list of attributes to ignore, for when we are evaluating parameters from
+        # a class of parameters
+        ignore = [
+            "__name__",
+            "__doc__",
+            "__package__",
+            "__loader__",
+            "__spec__",
+            "__file__",
+            "__cached__",
+            "__builtins__",
+            "absolute_import",
+            "division",
+            "print_function",
+            "unicode_literals",
+            "pybamm",
+            "constants",
+            "np",
+        ]
 
-    def __call__(self, t):
-        if self.type == "function":
-            return self.Crate(t) * self.capacity
-        elif self.type == "input":
-            return pybamm.InputParameter("C-rate") * self.capacity
+        # If 'parameters' is a class, extract the dict
+        if not isinstance(parameters, dict):
+            parameters = {
+                k: v for k, v in parameters.__dict__.items() if k not in ignore
+            }
+
+        evaluated_parameters = defaultdict(list)
+        # Calculate parameters for each C-rate
+        for Crate in [1, 10]:
+            # Update Crate
+            capacity = self.get("Cell capacity [A.h]")
+            if capacity is not None:
+                self.update(
+                    {"Current function [A]": Crate * capacity},
+                    check_already_exists=False,
+                )
+            for name, symbol in parameters.items():
+                if not callable(symbol):
+                    proc_symbol = self.process_symbol(symbol)
+                    if not (
+                        callable(proc_symbol)
+                        or proc_symbol.has_symbol_of_classes(
+                            (pybamm.Concatenation, pybamm.Broadcast)
+                        )
+                    ):
+                        evaluated_parameters[name].append(proc_symbol.evaluate(t=0))
+
+        # Calculate C-dependence of the parameters based on the difference between the
+        # value at 1C and the value at C / 10
+        for name, values in evaluated_parameters.items():
+            if values[1] == 0 or abs(values[0] / values[1] - 1) < 1e-10:
+                C_dependence = ""
+            elif abs(values[0] / values[1] - 10) < 1e-10:
+                C_dependence = " * Crate"
+            elif abs(values[0] / values[1] - 0.1) < 1e-10:
+                C_dependence = " / Crate"
+            evaluated_parameters[name] = (values[0], C_dependence)
+        # Print the evaluated_parameters dict to output_file
+        if output_file:
+            self.print_evaluated_parameters(evaluated_parameters, output_file)
+
+        return evaluated_parameters
+
+    def print_evaluated_parameters(self, evaluated_parameters, output_file):
+        """
+        Print a dictionary of evaluated parameters to an output file
+
+        Parameters
+        ----------
+        evaluated_parameters : defaultdict
+            The evaluated parameters, for further processing if needed
+        output_file : string, optional
+            The file to print parameters to. If None, the parameters are not printed,
+            and this function simply acts as a test that all the parameters can be
+            evaluated
+
+        """
+        # Get column width for pretty printing
+        column_width = max(len(name) for name in evaluated_parameters.keys())
+        s = "{{:>{}}}".format(column_width)
+        with open(output_file, "w") as file:
+            for name, (value, C_dependence) in sorted(evaluated_parameters.items()):
+                if 0.001 < abs(value) < 1000:
+                    file.write(
+                        (s + " : {:10.4g}{!s}\n").format(name, value, C_dependence)
+                    )
+                else:
+                    file.write(
+                        (s + " : {:10.3E}{!s}\n").format(name, value, C_dependence)
+                    )
+
+    @staticmethod
+    def find_parameter(path):
+        """Look for parameter file in the different locations
+        in PARAMETER_PATH
+        """
+        for location in pybamm.PARAMETER_PATH:
+            trial_path = os.path.join(location, path)
+            if os.path.isfile(trial_path):
+                return trial_path
+        raise FileNotFoundError("Could not find parameter {}".format(path))
