@@ -19,6 +19,8 @@ def is_notebook():
             return nb
         elif shell == "TerminalInteractiveShell":  # pragma: no cover
             return False  # Terminal running IPython
+        elif shell == "Shell":  # pragma: no cover
+            return True  # Google Colab notebook
         else:
             return False  # Other type (?)
     except NameError:
@@ -66,7 +68,7 @@ class Simulation:
         A list of variables to plot automatically
     C_rate: float (optional)
         The C_rate at which you would like to run a constant current
-        experiment at.
+        (dis)charge at.
     """
 
     def __init__(
@@ -82,25 +84,42 @@ class Simulation:
         quick_plot_vars=None,
         C_rate=None,
     ):
-        self._parameter_values = parameter_values or model.default_parameter_values
+        self.parameter_values = parameter_values or model.default_parameter_values
 
         if experiment is None:
-            self.operating_mode = "without experiment"
-            self.C_rate = C_rate
-            if self.C_rate:
-                self._parameter_values.update({"C-rate": self.C_rate})
+            # Check to see if the current is provided as data (i.e. drive cycle)
+            current = self._parameter_values.get("Current function [A]")
+            if isinstance(current, tuple):
+                self.operating_mode = "drive cycle"
+            else:
+                self.operating_mode = "without experiment"
+                if C_rate:
+                    self.C_rate = C_rate
+                    self._parameter_values.update(
+                        {
+                            "Current function [A]": self.C_rate
+                            * self._parameter_values["Cell capacity [A.h]"]
+                        }
+                    )
+
+            self._unprocessed_model = model
             self.model = model
         else:
             self.set_up_experiment(model, experiment)
 
         self.geometry = geometry or self.model.default_geometry
-        self._submesh_types = submesh_types or self.model.default_submesh_types
-        self._var_pts = var_pts or self.model.default_var_pts
-        self._spatial_methods = spatial_methods or self.model.default_spatial_methods
-        self._solver = solver or self.model.default_solver
-        self._quick_plot_vars = quick_plot_vars
+        self.submesh_types = submesh_types or self.model.default_submesh_types
+        self.var_pts = var_pts or self.model.default_var_pts
+        self.spatial_methods = spatial_methods or self.model.default_spatial_methods
+        self.solver = solver or self.model.default_solver
+        self.quick_plot_vars = quick_plot_vars
 
-        self.reset(update_model=False)
+        # Initialize empty built states
+        self._model_with_set_params = None
+        self._built_model = None
+        self._mesh = None
+        self._disc = None
+        self._solution = None
 
         # ignore runtime warnings in notebooks
         if is_notebook():  # pragma: no cover
@@ -117,14 +136,24 @@ class Simulation:
         time.
         """
         self.operating_mode = "with experiment"
-        self.model = model.new_copy(
-            options={
-                **model.options,
-                "operating mode": constant_current_constant_voltage_constant_power,
-            }
+
+        # Update model
+        new_model = model.new_copy(build=False)
+        new_model.submodels[
+            "external circuit"
+        ] = pybamm.external_circuit.FunctionControl(
+            new_model.param, constant_current_constant_voltage_constant_power
         )
+        new_model.submodels[
+            "experiment events"
+        ] = pybamm.external_circuit.ExperimentEvents(new_model.param)
+        new_model.build_model()
+        self._unprocessed_model = new_model
+        self.model = new_model
+
         if not isinstance(experiment, pybamm.Experiment):
             raise TypeError("experiment must be a pybamm `Experiment` instance")
+
         # Save the experiment
         self.experiment = experiment
         # Update parameter values with experiment parameters
@@ -206,76 +235,30 @@ class Simulation:
                 dt = 7 * 24 * 3600
             self._experiment_times.append(dt)
 
-        # add current and voltage events to the model
-        # current events both negative and positive to catch specification
-        n_cells = pybamm.electrical_parameters.n_cells
-        self.model.events.extend(
-            [
-                pybamm.Event(
-                    "Current cut-off (positive) [A] [experiment]",
-                    self.model.variables["Current [A]"]
-                    - abs(pybamm.InputParameter("Current cut-off [A]")),
-                ),
-                pybamm.Event(
-                    "Current cut-off (negative) [A] [experiment]",
-                    self.model.variables["Current [A]"]
-                    + abs(pybamm.InputParameter("Current cut-off [A]")),
-                ),
-                pybamm.Event(
-                    "Voltage cut-off [V] [experiment]",
-                    self.model.variables["Terminal voltage [V]"]
-                    - pybamm.InputParameter("Voltage cut-off [V]") / n_cells,
-                ),
-            ]
-        )
-
-    def set_defaults(self):
-        """
-        A method to set all the simulation specs to default values for the
-        supplied model.
-        """
-        self.geometry = self._model.default_geometry
-        self._parameter_values = self._model.default_parameter_values
-        self._submesh_types = self._model.default_submesh_types
-        self._var_pts = self._model.default_var_pts
-        self._spatial_methods = self._model.default_spatial_methods
-        self._solver = self._model.default_solver
-        self._quick_plot_vars = None
-
-    def reset(self, update_model=True):
-        """
-        A method to reset a simulation back to its unprocessed state.
-        """
-        if update_model:
-            self.model = self.model.new_copy(self._model_options)
-        self.geometry = copy.deepcopy(self._unprocessed_geometry)
-        self._model_with_set_params = None
-        self._built_model = None
-        self._mesh = None
-        self._disc = None
-        self._solution = None
-
     def set_parameters(self):
         """
-        A method to set the parameters in the model and the associated geometry. If
-        the model has already been built or solved then this will first reset to the
-        unprocessed state and then set the parameter values.
+        A method to set the parameters in the model and the associated geometry.
         """
 
         if self.model_with_set_params:
             return None
 
-        self._model_with_set_params = self._parameter_values.process_model(
-            self._model, inplace=True
-        )
-        self._parameter_values.process_geometry(self._geometry)
+        if self._parameter_values._dict_items == {}:
+            # Don't process if parameter values is empty
+            self._model_with_set_params = self._unprocessed_model
+        else:
+            self._model_with_set_params = self._parameter_values.process_model(
+                self._unprocessed_model, inplace=False
+            )
+            self._parameter_values.process_geometry(self._geometry)
+        self.model = self._model_with_set_params
 
     def build(self, check_model=True):
         """
         A method to build the model into a system of matrices and vectors suitable for
         performing numerical computations. If the model has already been built or
-        solved then this function will have no effect. If you want to rebuild,
-        first use "reset()". This method will automatically set the parameters
+        solved then this function will have no effect.
+        This method will automatically set the parameters
         if they have not already been set.
 
         Parameters
@@ -287,13 +270,16 @@ class Simulation:
 
         if self.built_model:
             return None
-
-        self.set_parameters()
-        self._mesh = pybamm.Mesh(self._geometry, self._submesh_types, self._var_pts)
-        self._disc = pybamm.Discretisation(self._mesh, self._spatial_methods)
-        self._built_model = self._disc.process_model(
-            self._model_with_set_params, inplace=False, check_model=check_model
-        )
+        elif self.model.is_discretised:
+            self._model_with_set_params = self.model
+            self._built_model = self.model
+        else:
+            self.set_parameters()
+            self._mesh = pybamm.Mesh(self._geometry, self._submesh_types, self._var_pts)
+            self._disc = pybamm.Discretisation(self._mesh, self._spatial_methods)
+            self._built_model = self._disc.process_model(
+                self._model_with_set_params, inplace=False, check_model=check_model
+            )
 
     def solve(
         self,
@@ -310,11 +296,21 @@ class Simulation:
         Parameters
         ----------
         t_eval : numeric type, optional
-            The times at which to compute the solution. If None and the parameter
-            "Current function [A]" is not read from data the model will
-            be solved for a full discharge (1 hour / C_rate). If None and the
-            parameter "Current function [A]" is read from data the model will be
-            solved at the times provided in the data.
+            The times (in seconds) at which to compute the solution. Can be
+            provided as an array of times at which to return the solution, or as a
+            list `[t0, tf]` where `t0` is the initial time and `tf` is the final time.
+            If provided as a list the solution is returned at 100 points within the
+            interval `[t0, tf]`.
+
+            If not using an experiment or running a drive cycle simulation (current
+            provided as data) `t_eval` *must* be provided.
+
+            If running an experiment the values in `t_eval` are ignored, and the
+            solution times are specified by the experiment.
+
+            If None and the parameter "Current function [A]" is read from data
+            (i.e. drive cycle simulation) the model will be solved at the times
+            provided in the data.
         solver : :class:`pybamm.BaseSolver`
             The solver to use to solve the model.
         external_variables : dict
@@ -333,12 +329,36 @@ class Simulation:
         if solver is None:
             solver = self.solver
 
-        if self.operating_mode == "without experiment":
-            # For drive cycles (current provided as data) we perform additional tests
-            # on t_eval (if provided) to ensure the returned solution captures the
-            # input. If the current is provided as data then the "Current function [A]"
-            # is the tuple (filename, data).
-            if isinstance(self._parameter_values["Current function [A]"], tuple):
+        if self.operating_mode in ["without experiment", "drive cycle"]:
+            # If t_eval is provided as [t0, tf] return the solution at 100 points
+            if isinstance(t_eval, list):
+                if len(t_eval) != 2:
+                    raise pybamm.SolverError(
+                        "'t_eval' can be provided as an array of times at which to "
+                        "return the solution, or as a list [t0, tf] where t0 is the "
+                        "initial time and tf is the final time, but has been provided "
+                        "as a list of length {}.".format(len(t_eval))
+                    )
+                else:
+                    t_eval = np.linspace(t_eval[0], t_eval[-1], 100)
+
+            if self.operating_mode == "without experiment":
+                if t_eval is None:
+                    raise pybamm.SolverError(
+                        "'t_eval' must be provided if not using an experiment or "
+                        "simulating a drive cycle. 't_eval' can be provided as an "
+                        "array of times at which to return the solution, or as a "
+                        "list [t0, tf] where t0 is the initial time and tf is the "
+                        "final time. "
+                        "For a constant current (dis)charge the suggested 't_eval'  "
+                        "is [0, 3700/C] where C is the C-rate."
+                    )
+
+            elif self.operating_mode == "drive cycle":
+                # For drive cycles (current provided as data) we perform additional
+                # tests on t_eval (if provided) to ensure the returned solution
+                # captures the input. If the current is provided as data then the
+                # "Current function [A]" is the tuple (filename, data).
                 filename = self._parameter_values["Current function [A]"][0]
                 time_data = self._parameter_values["Current function [A]"][1][:, 0]
                 # If no t_eval is provided, we use the times provided in the data.
@@ -347,10 +367,10 @@ class Simulation:
                         "Setting t_eval as specified by the data '{}'".format(filename)
                     )
                     t_eval = time_data
-                # If t_eval is provided we first check if it contains all of the times
-                # in the data to within 10-12. If it doesn't, we then check
-                # that the largest gap in t_eval is smaller than the smallest gap in the
-                # time data (to ensure the resolution of t_eval is fine enough).
+                # If t_eval is provided we first check if it contains all of the
+                # times in the data to within 10-12. If it doesn't, we then check
+                # that the largest gap in t_eval is smaller than the smallest gap in
+                # the time data (to ensure the resolution of t_eval is fine enough).
                 # We only raise a warning here as users may genuinely only want
                 # the solution returned at some specified points.
                 elif (
@@ -382,26 +402,23 @@ class Simulation:
                             ),
                             pybamm.SolverWarning,
                         )
-            # If not using a drive cycle and t_eval is not provided, set t_eval
-            # to correspond to a single discharge
-            elif t_eval is None:
-                C_rate = self._parameter_values["C-rate"]
-                if isinstance(C_rate, pybamm.InputParameter):
-                    C_rate = inputs["C-rate"]
-                try:
-                    t_end = 3600 / C_rate
-                except TypeError:
-                    t_end = 3600
-                t_eval = np.linspace(0, t_end, 100)
 
             self.t_eval = t_eval
-            self._solution = solver.solve(self.built_model, t_eval, inputs=inputs)
+            self._solution = solver.solve(
+                self.built_model,
+                t_eval,
+                external_variables=external_variables,
+                inputs=inputs,
+            )
 
         elif self.operating_mode == "with experiment":
             if t_eval is not None:
                 pybamm.logger.warning(
                     "Ignoring t_eval as solution times are specified by the experiment"
                 )
+            # Re-initialize solution, e.g. for solving multiple times with different
+            # inputs without having to build the simulation again
+            self._solution = None
             # Step through all experimental conditions
             inputs = inputs or {}
             pybamm.logger.info("Start running experiment")
@@ -426,14 +443,14 @@ class Simulation:
                     or "[experiment]" in self._solution.termination
                 ):
                     pybamm.logger.warning(
-                        """
-                        Experiment is infeasible: '{}' was triggered during '{}'. Try
-                        reducing current, shortening the time interval, or reducing
-                        the period.
-                        """.format(
-                            self._solution.termination,
-                            self.experiment.operating_conditions_strings[idx],
+                        "\n\n\tExperiment is infeasible: '{}' ".format(
+                            self._solution.termination
                         )
+                        + "was triggered during '{}'. ".format(
+                            self.experiment.operating_conditions_strings[idx]
+                        )
+                        + "Try reducing current, shortening the time interval, "
+                        "or reducing the period.\n\n"
                     )
                     break
             pybamm.logger.info(
@@ -442,6 +459,8 @@ class Simulation:
                 )
             )
 
+        return self.solution
+
     def step(
         self, dt, solver=None, npts=2, external_variables=None, inputs=None, save=True
     ):
@@ -457,7 +476,7 @@ class Simulation:
             The solver to use to solve the model.
         npts : int, optional
             The number of points at which the solution will be returned during
-            the step dt. default is 2 (returns the solution at t0 and t0 + dt).
+            the step dt. Default is 2 (returns the solution at t0 and t0 + dt).
         external_variables : dict
             A dictionary of external variables and their corresponding
             values at the current time. The variables must correspond to
@@ -542,9 +561,8 @@ class Simulation:
 
     @model.setter
     def model(self, model):
-        self._model = model
+        self._model = copy.copy(model)
         self._model_class = model.__class__
-        self._model_options = model.options
 
     @property
     def model_with_set_params(self):
@@ -554,31 +572,30 @@ class Simulation:
     def built_model(self):
         return self._built_model
 
-    @property
-    def model_options(self):
-        return self._model_options
-
     @property
     def geometry(self):
         return self._geometry
 
     @geometry.setter
     def geometry(self, geometry):
-        self._geometry = geometry
-        self._unprocessed_geometry = copy.deepcopy(geometry)
-
-    @property
-    def unprocessed_geometry(self):
-        return self._unprocessed_geometry
+        self._geometry = geometry.copy()
 
     @property
     def parameter_values(self):
         return self._parameter_values
 
+    @parameter_values.setter
+    def parameter_values(self, parameter_values):
+        self._parameter_values = parameter_values.copy()
+
     @property
     def submesh_types(self):
         return self._submesh_types
 
+    @submesh_types.setter
+    def submesh_types(self, submesh_types):
+        self._submesh_types = submesh_types.copy()
+
     @property
     def mesh(self):
         return self._mesh
@@ -587,17 +604,25 @@ class Simulation:
     def var_pts(self):
         return self._var_pts
 
+    @var_pts.setter
+    def var_pts(self, var_pts):
+        self._var_pts = var_pts.copy()
+
     @property
     def spatial_methods(self):
         return self._spatial_methods
 
+    @spatial_methods.setter
+    def spatial_methods(self, spatial_methods):
+        self._spatial_methods = spatial_methods.copy()
+
     @property
     def solver(self):
         return self._solver
 
     @solver.setter
     def solver(self, solver):
-        self._solver = solver
+        self._solver = solver.copy()
 
     @property
     def quick_plot_vars(self):
@@ -605,7 +630,7 @@ class Simulation:
 
     @quick_plot_vars.setter
     def quick_plot_vars(self, quick_plot_vars):
-        self._quick_plot_vars = quick_plot_vars
+        self._quick_plot_vars = copy.copy(quick_plot_vars)
 
     @property
     def solution(self):
@@ -613,7 +638,6 @@ class Simulation:
 
     def specs(
         self,
-        model_options=None,
         geometry=None,
         parameter_values=None,
         submesh_types=None,
@@ -623,68 +647,11 @@ class Simulation:
         quick_plot_vars=None,
         C_rate=None,
     ):
-        """
-        A method to set the various specs of the simulation. This method
-        automatically resets the model after the new specs have been set.
-
-        Parameters
-        ----------
-        model_options: dict, optional
-            A dictionary of options to tweak the model you are using
-        geometry: :class:`pybamm.Geometry`, optional
-            The geometry upon which to solve the model
-        parameter_values: dict, optional
-            A dictionary of parameters and their corresponding numerical
-            values
-        submesh_types: dict, optional
-            A dictionary of the types of submesh to use on each subdomain
-        var_pts: dict, optional
-            A dictionary of the number of points used by each spatial
-            variable
-        spatial_methods: dict, optional
-            A dictionary of the types of spatial method to use on each
-            domain (e.g. pybamm.FiniteVolume)
-        solver: :class:`pybamm.BaseSolver` (optional)
-            The solver to use to solve the model.
-        quick_plot_vars: list (optional)
-            A list of variables to plot automatically
-        C_rate: float (optional)
-            The C_rate at which you would like to run a constant current
-            experiment at.
-        """
-
-        if model_options:
-            self._model_options = model_options
-
-        if geometry:
-            self.geometry = geometry
-
-        if parameter_values:
-            self._parameter_values = parameter_values
-        if submesh_types:
-            self._submesh_types = submesh_types
-        if var_pts:
-            self._var_pts = var_pts
-        if spatial_methods:
-            self._spatial_methods = spatial_methods
-        if solver:
-            self._solver = solver
-        if quick_plot_vars:
-            self._quick_plot_vars = quick_plot_vars
-
-        if C_rate:
-            self.C_rate = C_rate
-            self._parameter_values.update({"C-rate": self.C_rate})
-
-        if (
-            model_options
-            or geometry
-            or parameter_values
-            or submesh_types
-            or var_pts
-            or spatial_methods
-        ):
-            self.reset()
+        "Deprecated method for setting specs"
+        raise NotImplementedError(
+            "The 'specs' method has been deprecated. "
+            "Create a new simulation for each different case instead."
+        )
 
     def save(self, filename):
         """Save simulation using pickle"""
